//
// $Id: WhereClause.java 760 2010-12-09 00:08:58Z samskivert $
//
// Depot library - a Java relational persistence library
// Copyright (C) 2006-2008 Michael Bayne and Pär Winzell
//
// This library is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

package com.samskivert.depot.clause;

import com.samskivert.depot.expression.SQLExpression;

import static com.google.common.base.Preconditions.checkArgument;

/**
 * Currently only exists as a type without any functionality of its own.
 */
public abstract class WhereClause implements QueryClause
{
    /**
     * Returns the condition associated with this where clause.
     */
    public abstract SQLExpression<?> getWhereExpression ();

    /**
     * Validates that the supplied persistent record type is the type matched by this where clause.
     * Not all clauses will be able to perform this validation, but those that can, should do so to
     * help alleviate programmer error.
     *
     * @exception IllegalArgumentException thrown if the supplied class is known not to by the type
     * matched by this where clause.
     */
    public void validateQueryType (Class<?> pClass)
    {
        // nothing by default
    }

    /**
     * A helper function for implementing {@link #validateQueryType}.
     */
    protected void validateTypesMatch (Class<?> qClass, Class<?> kClass)
    {
        checkArgument(qClass.equals(kClass),
                      "Class mismatch between persistent record and key in query " +
                      "[qtype=%s, ktype=%s].", qClass.getSimpleName(), kClass.getSimpleName());
    }
}
